#!/bin/bash

#--------------------------------------------------------#
# Tizen Interface Definition Langauge Compiler           #
#--------------------------------------------------------#

PATH=/bin:/usr/bin:/sbin:/usr/sbin

SCRIPT_FILE=$(readlink -f $0)
SCRIPT_DIR=$(dirname $SCRIPT_FILE)
BUILD_DIR=build

usage() {
	echo "Usage: $0 [command]"
	echo "Commands:"
	echo "    build              Build the package"
	echo "    build_win          Build the package for 64bit Windows"
	echo "    build_win32        Build the package for 32bit Windows"
	echo "    clean              Clean all artifacts"
	echo "    clean_build        Clean & Build the package"
}

cmd_build() {
	echo "[TIDL] Build the package"
	mkdir -p $SCRIPT_DIR/$BUILD_DIR
	cd $SCRIPT_DIR/$BUILD_DIR
	VER=`cat ../packaging/tidl.spec | grep 'Version' | sed 's/Version://' | tr -d ' \r'`
	cmake .. -DFULLVER=$VER -DMINIMUM_BUILD="yes"
	make clean
	make
}

cmd_build_win() {
	echo "[TIDL] Build the package"
	mkdir -p $SCRIPT_DIR/$BUILD_DIR
	cd $SCRIPT_DIR/$BUILD_DIR
	VER=`cat ../packaging/tidl.spec | grep 'Version' | sed 's/Version://' | tr -d ' \r'`
	cmake .. -DFULLVER=$VER -DMINIMUM_BUILD="yes" -DBUILD_WIN="yes"
	make clean
	make
}

cmd_build_win32() {
	echo "[TIDL] Build the package"
	mkdir -p $SCRIPT_DIR/$BUILD_DIR
	cd $SCRIPT_DIR/$BUILD_DIR
	VER=`cat ../packaging/tidl.spec | grep 'Version' | sed 's/Version://' | tr -d ' \r'`
	cmake .. -DFULLVER=$VER -DMINIMUM_BUILD="yes" -DBUILD_WIN="yes" -DBUILD_32="yes"
	make clean
	make
}

cmd_clean() {
	echo "[TIDL] Clean all artifacts"
	cd $SCRIPT_DIR/$BUILD_DIR
	make clean
	cd ..
	rm -rf $SCRIPT_DIR/$BUILD_DIR
	rm -rf $SCRIPT_DIR/idlc/location*
	rm -rf $SCRIPT_DIR/idlc/position*
}

cmd_clean_build() {
	cmd_clean
	cmd_build
}

cmd=$1; shift;
case "$cmd" in
	build|--build|-b) cmd_build $@ ;;
	build_win|--build_win|-bw) cmd_build_win $@ ;;
	build_win32|--build_win32|-bw32) cmd_build_win32 $@ ;;
	clean|--clean|-c) cmd_clean $@ ;;
	clean_build |--clean_build |-cb)  cmd_clean_build $@ ;;
	*)     usage ;;

esac
