/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <utility>

#include "idlc/attribute.h"

namespace tidl {

Attribute::Attribute(std::string key, std::string value, unsigned line)
    : key_(std::move(key)), value_(std::move(value)), line_(line) {
}

const std::string& Attribute::GetKey() const {
  return key_;
}

const std::string& Attribute::GetValue() const {
  return value_;
}

const unsigned Attribute::GetLine() const {
  return line_;
}

void Attributes::Add(Attribute* attr) {
  attrs_.emplace_back(attr);
}

const std::list<std::unique_ptr<Attribute>>& Attributes::GetAttrs() const {
  return attrs_;
}

bool Attributes::Exist(Attribute* attr) const {
  for (auto& a : attrs_) {
    if (a->GetKey() == attr->GetKey() &&
        a->GetValue() == attr->GetValue())
      return true;
  }

  return false;
}

}  // namespace tidl
