/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef IDLC_CS_GEN_CS_GEN_BASE_H_
#define IDLC_CS_GEN_CS_GEN_BASE_H_

#include <memory>
#include <string>
#include <map>

#include "idlc/type.h"
#include "idlc/structure.h"
#include "idlc/generator.h"

namespace tidl {

class CsGeneratorBase : public Generator {
 public:
  explicit CsGeneratorBase(std::shared_ptr<Document> doc);
  virtual ~CsGeneratorBase() = default;

  void GenCopyright(std::ofstream& stream);
  void GenStructures(std::ofstream& stream);
  void GenStructure(std::ofstream& stream, const Structure& st);
  void GenSerializer(std::ofstream& stream);
  void GenSerializer(std::ofstream& stream, const Structure& st);
  void GenListSerializer(std::ofstream& stream);
  void GenListSerializer(std::ofstream& stream, const BaseType& type);
  void GenMethodId(std::ofstream& stream, const Interface& iface);
  void GenDelegateId(std::ofstream& stream, const Interface& iface);
  void GenDeclaration(std::ofstream& stream,
                      const Declaration& decl, bool semicol = true);
  void GenParameters(std::ofstream& stream, const Parameters& ps);
  void GenCallbacks(std::ofstream& stream, const Interface& iface,
                    bool is_proxy);

  std::string ConvertTypeToString(const BaseType& type);
  std::string ConvertTypeToDeserializer(const BaseType& type,
                                        std::string id, std::string parcel,
                                        bool make_new_type = true,
                                        const std::string iface_id = "");
  std::string ConvertTypeToSerializer(const BaseType& type,
                                      std::string id, std::string parcel,
                                      const std::string iface_id = "");
  std::string ConvertTypeToParcelType(const std::string& key);
  std::string GetParameters(const Parameters& ps);
  std::string Tab(int cnt);
  std::string NLine(int cnt);

 protected:
  const int TAB_SIZE = 4;

 private:
  void GenWriteBundle(std::ofstream& stream, const std::string& id);
  void AddSerializerList(const BaseType& type);
  void GenCallback(std::ofstream& stream, const Declaration& decl,
                   const std::string& id, bool is_proxy);
  void GenReceivedEvent(std::ofstream& stream, const Declaration& decl,
                        const std::string& id);
  void GenInvokeMethod(std::ofstream& stream, const Declaration& decl,
                       const std::string& id);

 private:
  std::map<std::string, std::string> type_map_;
  std::map<std::string, std::string> parcel_type_map_;
  std::map<std::string, const BaseType*> serializer_list_;
};

}  // namespace tidl

#endif  // IDLC_CS_GEN_CS_GEN_BASE_H_
