/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <gtest/gtest.h>

#include <iostream>

#include "idlc/block.h"

class BlockTest : public testing::Test {
 public:
  tidl::Block* testBlock;

  virtual void SetUp() {
    testBlock = new tidl::Block("TestBlock", tidl::Block::TYPE_INTERFACE,
        "", 28);
  }
  virtual void TearDown() {
    delete testBlock;
  }
};

TEST_F(BlockTest, Block_Constructor) {
  tidl::Block* block = new tidl::Block("StructureBlock",
      tidl::Block::TYPE_STRUCTURE, "", __LINE__);
  EXPECT_NE(block, nullptr);
  delete block;
}

TEST_F(BlockTest, Block_GetID) {
  EXPECT_EQ(testBlock->GetID(), "TestBlock");
}

TEST_F(BlockTest, Block_GetType) {
  EXPECT_EQ(testBlock->GetType(), tidl::Block::TYPE_INTERFACE);
}

TEST_F(BlockTest, Block_GetLine) {
  EXPECT_EQ(testBlock->GetLine(), 28);
}

TEST_F(BlockTest, Block_GetComments) {
  std::string comments = "Test Block";
  tidl::Block* block = new tidl::Block("StructureBlock",
      tidl::Block::TYPE_STRUCTURE, comments, __LINE__);
  EXPECT_NE(block, nullptr);
  EXPECT_EQ(block->GetComments(), comments);
  delete block;
}
