/*
 * Copyright (c) 2017 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <gtest/gtest.h>

#include <iostream>
#include <memory>

#include "idlc/generator.h"
#include "idlc/interface.h"
#include "idlc/structure.h"
#include "idlc/attribute.h"

class SampleGenerator : public tidl::Generator {
 public:
  explicit SampleGenerator(std::shared_ptr<tidl::Document> doc)
      : tidl::Generator(doc), count_(0) {}
  ~SampleGenerator() {}

  void OnInitGen(std::ofstream& stream) override {
    count_++;
  }
  void OnFiniGen(std::ofstream& stream) override {
    count_++;
  }

  int GetCount() {
    return count_;
  }

 private:
  int count_;
};

class GeneratorTest : public testing::Test {
 public:
  SampleGenerator* gen;

  virtual void SetUp() {
    gen = new SampleGenerator(
        std::shared_ptr<tidl::Document>(new tidl::Document()));
  }
  virtual void TearDown() {
    delete gen;
  }
};

TEST_F(GeneratorTest, Generator_Constructor) {
  EXPECT_NE(gen, nullptr);
}

TEST_F(GeneratorTest, Generator_Run) {
  EXPECT_NE(gen, nullptr);
  gen->Run("test.out");
  EXPECT_EQ(gen->GetCount(), 2);
}
